<?php
/**
 * Core plugin class.
 *
 * @link       https://www.gracethemes.com
 * @since      1.0.0
 *
 * @package    Easy_GTBlocks
 * @subpackage Easy_GTBlocks/includes
 */

/**
 * The core plugin class Easy_GTBlocks.
 *
 * @since      1.0.0
 * @package    Easy_GTBlocks
 * @subpackage Easy_GTBlocks/includes
 * @author     Grace Themes <support@gracethemes.com>
 */
class Easy_GTBlocks {

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'EASY_GTBLOCKS_VER' ) ) {
			$this->version = EASY_GTBLOCKS_VER;
		} else {
			$this->version = '1.0';
		}
		$this->plugin_name = 'easy-gtblocks';

		add_action( 'plugins_loaded', array( $this, 'EASY_GTBLOCKS_loader' ) );

		add_action( 'init', array( $this, 'EASY_GTBLOCKS_block_assets' ) );

		add_filter( 'block_categories_all', array( $this, 'EASY_GTBLOCKS_add_custom_block_category'), 10, 2 );

		add_action( 'enqueue_block_editor_assets', array( $this, 'EASY_GTBLOCKS_editor_assets' ) );

		add_action( 'enqueue_block_assets', array( $this, 'EASY_GTBLOCKS_frontend_assets' ) );

		add_action( 'admin_enqueue_scripts', array( &$this, 'EASY_GTBLOCKS_admin_enqueue_styles' ) );

		// Easy GT Blocks Menu.
		add_action( 'admin_menu', array( $this, 'EASY_GTBLOCKS_admin_menu' ) );

		// Remove all admin notices from specific pages.
		add_action( 'admin_init', array( $this, 'EASY_GTBLOCKS_admin_init' ) );

		// Redirect to Getting Started Page on Plugin Activation.
		add_action( 'admin_init', array( $this, 'EASY_GTBLOCKS_maybe_redirect_to_getting_started' ) );

		add_action( 'wp_ajax_responsive_block_editor_post_pagination', array( $this, 'post_pagination' ) );
        add_action( 'wp_enqueue_scripts', array( $this, 'load_dashicons_front_end' ) );

	}

	/**
	 * Sends the Post pagination markup to edit.js
	 *
	 * @since 1.0.0
	 */
	public function post_pagination() {
		check_ajax_referer( 'responsive_block_editor_ajax_nonce', 'nonce' );

		if ( isset( $_POST['attributes'] ) ) {

			$query = $this->get_query( $_POST['attributes'], 'grid' );

			$pagination_markup = $this->render_pagination( $query, $_POST['attributes'] );

			wp_send_json_success( $pagination_markup );
		}

		wp_send_json_error( ' No attributes recieved' );
	}

	/**
	 * Renders the post post pagination on server.
	 *
	 * @param object $query WP_Query object.
	 * @param array  $attributes Array of block attributes.
	 * @since 1.0.0
	 */
	public function render_pagination( $query, $attributes ) {

		$permalink_structure = get_option( 'permalink_structure' );
		$base                = untrailingslashit( wp_specialchars_decode( get_pagenum_link() ) );
		$base                = $this->build_base_url( $permalink_structure, $base );
		$format              = $this->paged_format( $permalink_structure, $base );
		$paged               = $this->get_paged( $query );
		$page_limit          = min( $attributes['pageLimit'], $query->max_num_pages );
		$page_limit          = isset( $page_limit ) ? $page_limit : $attributes['postsToShow'];
		$attributes['postsToShow'];

		$links = paginate_links(
			array(
				'base'      => $base . '%_%',
				'format'    => $format,
				'current'   => ( ! $paged ) ? 1 : $paged,
				'total'     => $page_limit,
				'type'      => 'array',
				'mid_size'  => 4,
				'end_size'  => 4,
				'prev_text' => $attributes['previousButtonText'],
				'next_text' => $attributes['nextButtonText'],
			)
		);

		if ( isset( $links ) ) {
			return wp_kses_post( implode( PHP_EOL, $links ) );
		}

		return '';
	}

	/**
	 * Gives the paged Query var.
	 *
	 * @param Object $query Query.
	 * @return int $paged Paged Query var.
	 * @since 1.0.0
	 */
	public static function get_paged( $query ) {

		global $paged;

		// Check the 'paged' query var.
		$paged_qv = $query->get( 'paged' );

		if ( is_numeric( $paged_qv ) ) {
			return $paged_qv;
		}

		// Check the 'page' query var.
		$page_qv = $query->get( 'page' );

		if ( is_numeric( $page_qv ) ) {
			return $page_qv;
		}

		// Check the $paged global?
		if ( is_numeric( $paged ) ) {
			return $paged;
		}

		return 0;
	}


	/**
	 * Returns the Paged Format.
	 *
	 * @param string $permalink_structure Premalink Structure.
	 * @param string $base Base.
	 * @since 1.0.0
	 */
	public static function paged_format( $permalink_structure, $base ) {

		$page_prefix = empty( $permalink_structure ) ? 'paged' : 'page';

		if ( ! empty( $permalink_structure ) ) {
			$format  = substr( $base, -1 ) !== '/' ? '/' : '';
			$format .= $page_prefix . '/';
			$format .= '%#%';
			$format .= substr( $permalink_structure, -1 ) === '/' ? '/' : '';
		} elseif ( empty( $permalink_structure ) || is_search() ) {
			$parse_url = wp_parse_url( $base, PHP_URL_QUERY );
			$format    = empty( $parse_url ) ? '?' : '&';
			$format   .= $page_prefix . '=%#%';
		}

		return $format;
	}

	/**
	 * Builds the base url.
	 *
	 * @param string $permalink_structure Premalink Structure.
	 * @param string $base Base.
	 * @since 1.0.0
	 */
	public static function build_base_url( $permalink_structure, $base ) {
		// Check to see if we are using pretty permalinks.
		if ( ! empty( $permalink_structure ) ) {

			if ( strrpos( $base, 'paged-' ) ) {
				$base = substr_replace( $base, '', strrpos( $base, 'paged-' ), strlen( $base ) );
			}

			// Remove query string from base URL since paginate_links() adds it automatically.
			// This should also fix the WPML pagination issue that was added since 1.0.3.
			if ( count( $_GET ) > 0 ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended
				$base = strtok( $base, '?' );
			}

			// Add trailing slash when necessary.
			if ( '/' === substr( $permalink_structure, -1 ) ) {
				$base = trailingslashit( $base );
			} else {
				$base = untrailingslashit( $base );
			}
		} else {
			$url_params = wp_parse_url( $base, PHP_URL_QUERY );

			if ( empty( $url_params ) ) {
				$base = trailingslashit( $base );
			}
		}

		return $base;
	}

	/**
	 * Returns Query.
	 *
	 * @param array  $attributes The block attributes.
	 * @param string $block_type The Block Type.
	 * @since 1.0.0
	 */
	public static function get_query( $attributes, $block_type ) {

		// Block type is grid/masonry/carousel/timeline.
		$query_args = array(
			'posts_per_page'      => ( isset( $attributes['postsToShow'] ) ) ? $attributes['postsToShow'] : 6,
			'post_status'         => 'publish',
			'post_type'           => ( isset( $attributes['postType'] ) ) ? $attributes['postType'] : 'post',
			'order'               => ( isset( $attributes['order'] ) ) ? $attributes['order'] : 'desc',
			'orderby'             => ( isset( $attributes['orderBy'] ) ) ? $attributes['orderBy'] : 'date',
			'ignore_sticky_posts' => 1,
			'paged'               => 1,
		);

		if ( $attributes['excludeCurrentPost'] ) {
			$query_args['post__not_in'] = array( get_the_ID() );
		}

		if ( isset( $attributes['categories'] ) && '' !== $attributes['categories'] ) {
			$query_args['tax_query'][] = array(
				'taxonomy' => ( isset( $attributes['taxonomyType'] ) ) ? $attributes['taxonomyType'] : 'category',
				'field'    => 'id',
				'terms'    => $attributes['categories'],
				'operator' => 'IN',
			);
		}

		if ( 'grid' === $block_type && isset( $attributes['postPagination'] ) && true === $attributes['postPagination'] ) {

			if ( get_query_var( 'paged' ) ) {

				$paged = get_query_var( 'paged' );

			} elseif ( get_query_var( 'page' ) ) {

				$paged = get_query_var( 'page' );

			} else {

				$paged = 1;

			}
			$query_args['posts_per_page'] = $attributes['postsToShow'];
			$query_args['paged']          = $paged;

		}

		if ( 'masonry' === $block_type && isset( $attributes['paginationType'] ) && 'none' !== $attributes['paginationType'] && isset( $attributes['paged'] ) ) {

			$query_args['paged'] = $attributes['paged'];

		}

		$query_args = apply_filters( "responsive_block_editor_post_query_args_{$block_type}", $query_args, $attributes );

		return new WP_Query( $query_args );
	}

	/**
	 * Register the menu for the plugin.
	 *
	 * @return void [description]
	 */
	public function EASY_GTBLOCKS_admin_menu() {
		// Create Sub Menu with parent slug null .
		add_submenu_page(
			'options-general.php',
			__( 'Easy GT Blocks', 'easy-gtblocks' ),
			__( 'Easy GT Blocks', 'easy-gtblocks' ),
			'manage_options',
			'Easy_GTBlocks',
			array( $this, 'EASY_GTBLOCKS_getting_started' ),
			75
		);

	}

	/**
	 * Display Getting Started Page.
	 *
	 * Output the content for the getting started page.
	 *
	 * @access public
	 */
	public function EASY_GTBLOCKS_getting_started() {

		?>
		<div class="easy-gtblocks-admin-page gt-welcome-wrap">
			<div class="gt-welcome-promo-container">
				<div class="gt-welcome-wrap-block space50">					 
					<div class="gt-info-listing">
						<h3><?php echo esc_html__( 'Welcome to Easy GT Blocks', 'easy-gtblocks' ); ?></h3>
						<p class="easy-gtblocks-subtitle">
						</p>
						<p class="easy-gtblocks-subtitle">
							<?php echo esc_html__( 'Working with the Easy GT Blocks is just like working with the default WordPress blocks. They all offer a quick way to add various types of content to your posts and pages. Easy GT Blocks provides advanced tools for creating a websites faster.', 'easy-gtblocks' ); ?>
						</p>
					</div>				
					<div class="gt-info-listing">
						<h3 class="sub-title"><?php echo esc_html__( 'Manage Your Blocks As Your Preferences', 'easy-gtblocks' ); ?></h3>
					</div>
<div class="gt-info-listing column-3BX">
<?php
$feature_list_array = array(
array( 'Advanced Columns Block', 'The Advanced Columns block gives you a powerful, flexible column system to build custom, full-page layouts for your posts and pages', 'fsc-icon01.png' ),
array( 'Info Box', 'Create a box containing an icon or image and, optionally, a title, description, and learn more text. Style static and hover separately', 'fsc-icon02.png' ),
array( 'Container Block', 'The Container block allows you to wrap several individual blocks inside a parent container to create differently styled sections of content', 'fsc-icon03.png' ),
array( 'Advanced Button', 'Create an advanced button or a row of buttons. Style each one including hover controls plus you can use an icon and display them side by side', 'fsc-icon04.png' ),
array( 'Post and Page Grid Block', 'The Post and Page Grid block gives you an advanced, customizable, and sortable grid of posts and pages for your site', 'fsc-icon05.png' ),
array( 'Advanced Text', 'Create a heading or paragraph and define sizes for desktop, tablet and mobile along with font family, colors, etc.', 'fsc-icon06.png' ),
array( 'Icon', 'Add beautiful icons to your lists, checklists, feature lists, or any other list and make them more engaging', 'fsc-icon07.png' ),
array( 'Call To Action', 'The Call to Action block allows you to add a compact, wide, or full-width call-to-action section to your post or page', 'fsc-icon08.png' ),
array( 'Image Gallery', 'Photo galleries, carousels, and sliders! Enable custom links, captions, and more. Plus, you can select image sizes', 'fsc-icon09.png' ),
array( 'Spacer/Divider', 'Easily create a divider and determine the space around it or just create some space in your content', 'fsc-icon10.png' ),
array( 'Countdown', 'Increase your conversions by adding a sense of urgency to your offering. Pro includes evergreen campaigns as well', 'fsc-icon11.png' ),
array( 'Tabs', 'Create custom vertical or horizontal tabs with advanced styling controls. Each tab content is an empty canvas able to contain any other blocks', 'fsc-icon12.png' ),
array( 'Accordion', 'The Accordion block allows you to display content in expandable tabs so you can easily add lots of content without increasing page length', 'fsc-icon13.png' ),
array( 'Testimonial Block', 'Create confidence in your brand or product by showing off beautiful unique testimonials, add as a carousel or a grid', 'fsc-icon14.png' ),
array( 'Pricing Block', 'The Pricing block allows you to build beautiful, dynamic, responsive pricing tables with settings to change columns, colors, padding, and position', 'fsc-icon15.png' ),
);

foreach ( $feature_list_array as $single_feature ) {

?>
<div class="gt-welcome-promo">
    <div class="gt-welcome-promo-icon">
        <img src="<?php echo esc_url( EASY_GTBLOCKS_URL . 'assets/images/' . $single_feature[2] ); ?>">
    </div>
    <div class="gt-welcome-promo-info">
        <h4>
            <?php
                /* translators: %s: search term */
                $title = sprintf( __( '%s', 'easy-gtblocks' ), $single_feature[0] );
                echo esc_html( $title );
            ?>
        </h4>
        <p>
            <?php
                /* translators: %s: search term */
                $short_desc = sprintf( __( '%s.', 'easy-gtblocks' ), $single_feature[1] );
                echo esc_html( $short_desc );
            ?>
        </p>
    </div>
</div>
<?php } ?>
</div>					
				</div>			
		</div>		 
		<?php 
	}

	/**
	 * EASY_GTBLOCKS_maybe_redirect_to_getting_started description
	 *
	 * @return [type] [description]
	 */
	public function EASY_GTBLOCKS_maybe_redirect_to_getting_started() {
		if ( ! get_transient( 'EASY_GTBLOCKS_activation_redirect' ) ) {
			return;
		}

		if ( wp_doing_ajax() ) {
			return;
		}

		delete_transient( 'EASY_GTBLOCKS_activation_redirect' );

		if ( is_network_admin() || isset( $_GET['activate-multi'] ) ) {
			return;
		}

		wp_safe_redirect( admin_url( 'admin.php?page=Easy_GTBlocks' ) );

		exit;
	}

	/**
	 * Initialize the blocks
	 *
	 * @since    1.0.0
	 */
	public function EASY_GTBLOCKS_loader() {
		/**
		* Load Post Grid PHP
		*/
		require_once EASY_GTBLOCKS_DIR . 'src/blocks/post-grid/index.php';
		require_once EASY_GTBLOCKS_DIR . 'src/blocks/post-carousel/index.php';
		require_once EASY_GTBLOCKS_DIR . 'src/blocks/gallery-masonry/index.php';
		require_once EASY_GTBLOCKS_DIR . 'src/blocks/accordion/index.php';
		require_once EASY_GTBLOCKS_DIR . 'src/blocks/post-timeline/index.php';
		require_once EASY_GTBLOCKS_DIR . 'src/blocks/image-slider/index.php';
		require_once EASY_GTBLOCKS_DIR . 'src/blocks/testimonial-slider/index.php';
		require_once EASY_GTBLOCKS_DIR . 'src/utils/fonts.php';
	}

	/** Adds the Easy GT Blocks block category.
	 *
	 * @param array $categories Existing block categories.
	 *
	 * @return array Updated block categories.
	 */
	public function EASY_GTBLOCKS_add_custom_block_category( $categories ) {
		return array_merge(
			array(
				array(
					'slug'  => 'Easy_GTBlocks',
					'title' => __( 'Easy GT Blocks', 'easy-gtblocks' ),
					'icon'  => 'screenoptions',
				),
			), $categories
		);
	}


	/**
	 * Get Post Types.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public static function get_post_types() {

		$post_types = get_post_types(
			array(
				'public'       => true,
				'show_in_rest' => true,
			),
			'objects'
		);

		$options = array();

		foreach ( $post_types as $post_type ) {
			if ( 'product' === $post_type->name ) {
				continue;
			}

			if ( 'attachment' === $post_type->name ) {
				continue;
			}

			$options[] = array(
				'value' => $post_type->name,
				'label' => $post_type->label,
			);
		}

		return $options;
	}


	/**
	 * Enqueue assets for backend editor
	 *
	 * @since 1.0.0
	 */
	public function EASY_GTBLOCKS_editor_assets() {
		$responsive_block_editor_ajax_nonce = wp_create_nonce( 'responsive_block_editor_ajax_nonce' );

		// Load the compiled blocks into the editor.
		wp_enqueue_script(
			'Easy_GTBlocks-block-js',
			EASY_GTBLOCKS_URL . '/dist/easy-gtblocks.js',
			array( 'lodash', 'react', 'react-dom', 'wp-api-fetch', 'wp-blob', 'wp-block-editor', 'wp-blocks', 'wp-components', 'wp-compose', 'wp-data', 'wp-date', 'wp-dom-ready', 'wp-edit-post', 'wp-editor', 'wp-element', 'wp-hooks', 'wp-i18n', 'wp-keycodes', 'wp-plugins', 'wp-polyfill', 'wp-rich-text', 'wp-token-list', 'wp-url', 'jquery' ),
			filemtime( EASY_GTBLOCKS_DIR . 'dist/easy-gtblocks.js' ),
			true
		);

		$user_data = wp_get_current_user();
		unset( $user_data->user_pass, $user_data->user_email );

		// Pass in REST URL.
		wp_localize_script(
			'Easy_GTBlocks-block-js',
			'responsive_globals',
			array(
				'rest_url'                           => esc_url( rest_url() ),
				'ajax_url'                           => admin_url( 'admin-ajax.php' ),
				'user_data'                          => $user_data,
				'pro_activated'                      => false,
				'is_wpe'                             => function_exists( 'is_wpe' ),
				'post_types'                         => $this->get_post_types(),
				'all_taxonomy'                       => $this->get_related_taxonomy(),
				'responsive_block_editor_ajax_nonce' => $responsive_block_editor_ajax_nonce,
			)
		);

		// Load the compiled styles into the editor.
		wp_enqueue_style(
			'Easy_GTBlocks-block-editor-css',
			EASY_GTBLOCKS_URL . 'dist/easy-gtblocks-editor.css',
			array( 'wp-edit-blocks' ),
			filemtime( EASY_GTBLOCKS_DIR . 'dist/easy-gtblocks-editor.css' )
		);
	}

	/**
	 * Get all taxonomies.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public static function get_related_taxonomy() {

		$post_types = self::get_post_types();

		$return_array = array();

		foreach ( $post_types as $key => $value ) {
			$post_type = $value['value'];

			$taxonomies = get_object_taxonomies( $post_type, 'objects' );
			$data       = array();

			foreach ( $taxonomies as $tax_slug => $tax ) {
				if ( ! $tax->public || ! $tax->show_ui || ! $tax->show_in_rest ) {
					continue;
				}

				$data[ $tax_slug ] = $tax;

				$terms = get_terms( $tax_slug );

				$related_tax = array();

				if ( ! empty( $terms ) ) {
					foreach ( $terms as $t_index => $t_obj ) {
						$related_tax[] = array(
							'id'   => $t_obj->term_id,
							'name' => $t_obj->name,
						);
					}

					$return_array[ $post_type ]['terms'][ $tax_slug ] = $related_tax;
				}
			}

			$return_array[ $post_type ]['taxonomy'] = $data;
		}

		return $return_array;
	}
	/**
	 * Enqueue assets for frontend
	 *
	 * @since 1.0.0
	 */
	public function EASY_GTBLOCKS_frontend_assets() {

		// Load the compiled blocks into the editor.
		wp_enqueue_script(
			'Easy_GTBlocks-frontend-js',
			EASY_GTBLOCKS_URL . '/dist/frontend_blocks.js',
			array('jquery'),
			filemtime( EASY_GTBLOCKS_DIR . 'dist/frontend_blocks.js' ),
			true
		);

	}


	/**
	 * Enqueue assets for frontend and backend
	 *
	 * @since 1.0.0
	 */
	public function EASY_GTBLOCKS_block_assets() {

		// Load the compiled styles.
		wp_enqueue_style(
			'Easy_GTBlocks-style-css',
			EASY_GTBLOCKS_URL . 'dist/easy-gtblocks-style.css',
			array(),
			filemtime( EASY_GTBLOCKS_DIR . 'dist/easy-gtblocks-style.css' )
		);
	}

	/**
	 * Include Admin css
	 *
	 * @return void [description]
	 */
	public function EASY_GTBLOCKS_admin_enqueue_styles() {
		// Responsive Ready Sites admin styles.
		wp_register_style( 'easy-gtblocks-admin', EASY_GTBLOCKS_URL . 'assets/css/easy-gtblocks-admin.css', false, '1.0.0' );
		wp_enqueue_style( 'easy-gtblocks-admin' );
		wp_enqueue_script(
			'easy-gtblocks-admin-jsfile',
			EASY_GTBLOCKS_URL . 'assets/js/easy-gtblocks-admin.js',
			array( 'jquery' ),
			'1.0.0',
			true
		);
	}

	/**
	 * On admin init.
	 *
	 * Preform actions on WordPress admin initialization.
	 *
	 * Fired by `admin_init` action.
	 *
	 * @access public
	 */
	public function EASY_GTBLOCKS_admin_init() {

		$this->EASY_GTBLOCKS_remove_all_admin_notices();
	}

	/**
	 * [EASY_GTBLOCKS_remove_all_admin_notices description]
	 */
	private function EASY_GTBLOCKS_remove_all_admin_notices() {
		$EASY_GTBLOCKS_pages = array(
			'Easy_GTBlocks',
			'easy-gtblocks',
		);

		if ( empty( $_GET['page'] ) || ! in_array( $_GET['page'], $EASY_GTBLOCKS_pages, true ) ) {
			return;
		}

		remove_all_actions( 'admin_notices' );
	}

    /**
    * Adding Dashicons in WordPress Front-end
    */
    public function load_dashicons_front_end() {
        wp_enqueue_style( 'dashicons' );
    }

}
