import fontOptions from "../../../utils/googlefonts";
import { loadGoogleFont } from "../../../utils/font";

/**
 * Inspector Controls
 */

// Setup the block
const { __ } = wp.i18n;
const { select } = wp.data;

const { Component, Fragment } = wp.element;

// Import block components
const {
  InspectorControls,
  PanelColorSettings,
  AlignmentToolbar,
  BlockControls,
  InnerBlocks,
} = wp.editor;

// Import Inspector components
const {
  PanelBody,
  RangeControl,
  SelectControl,
  BaseControl,
  ToggleControl,
  TabPanel,
  Dashicon,
} = wp.components;

/**
 * Create an Inspector Controls wrapper Component
 */
export default class Inspector extends Component {
  constructor(props) {
    super(...arguments);
  }

  componentDidMount() {
    // Pushing Style tag for this block css.
    const $style = document.createElement("style");
    $style.setAttribute(
      "id",
      "easy-gtblocks-style-icon-list-" + this.props.clientId
    );
    document.head.appendChild($style);

    this.changeChildAttr(this.props.attributes.hideLabel);
  }

  changeChildAttr(value) {
    const { setAttributes } = this.props;
    const getChildBlocks = select("core/block-editor").getBlocks(
      this.props.clientId
    );

    getChildBlocks.forEach((iconChild, key) => {
      iconChild.attributes.hideLabel = value;
    });
    setAttributes({ hideLabel: value });
  }

  render() {
    // Setup the attributes
    const {
      attributes: {
        align,
        icon_count,
        icons,
        gap,
        inner_gap,
        icon_layout,
        iconPosition,
        size,
        hideLabel,
        borderRadius,
        bgSize,
        border,
        fontSize,
        labelFontFamily,
        labelFontWeight,
        labelFontLineHeight,
        labelFontSize,
        labelFontSizeTablet,
        labelFontSizeMobile,
      },
      setAttributes,
    } = this.props;

    // Font Weight Options
    const fontWeightOptions = [
      {
        value: "100",
        label: __("100", "easy-gtblocks"),
      },
      {
        value: "200",
        label: __("200", "easy-gtblocks"),
      },
      {
        value: "300",
        label: __("300", "easy-gtblocks"),
      },
      {
        value: "400",
        label: __("400", "easy-gtblocks"),
      },
      {
        value: "500",
        label: __("500", "easy-gtblocks"),
      },
      {
        value: "600",
        label: __("600", "easy-gtblocks"),
      },
      {
        value: "700",
        label: __("700", "easy-gtblocks"),
      },
      {
        value: "800",
        label: __("800", "easy-gtblocks"),
      },
      {
        value: "900",
        label: __("900", "easy-gtblocks"),
      },
    ];

    const labelClass = hideLabel
      ? "easy-gtblocks-icon-list__no-label"
      : "";

    return (
      <Fragment>
        <InspectorControls>
          <PanelBody
            title={__("General", "easy-gtblocks")}
            initialOpen={true}
          >
            <SelectControl
              label={__("Layout")}
              value={icon_layout}
              options={[
                {
                  value: "horizontal",
                  label: __("Horizontal", "easy-gtblocks"),
                },
                {
                  value: "vertical",
                  label: __("Vertical", "easy-gtblocks"),
                },
              ]}
              onChange={(value) => setAttributes({ icon_layout: value })}
            />

            <ToggleControl
              label={__("Hide Labels", "easy-gtblocks")}
              checked={hideLabel}
              onChange={(value) => this.changeChildAttr(value)}
            />
            <hr className="easy-gtblocks-editor__separator" />
            <RangeControl
              label={__("Gap between Items", "easy-gtblocks")}
              value={gap}
              onChange={(value) => setAttributes({ gap: value })}
              help={__(
                "Note: For better editing experience, the gap between items might look larger than applied.  Viewing in frontend will show the actual results."
              )}
              min={0}
              max={100}
            />
            {!hideLabel && (
              <Fragment>
                <SelectControl
                  label={__("Font Family", "easy-gtblocks")}
                  options={fontOptions}
                  value={labelFontFamily}
                  onChange={(value) => {
                    setAttributes({
                      labelFontFamily: value,
                    }),
                      loadGoogleFont(value);
                  }}
                />
                <TabPanel
                  className=" responsive-size-type-field-tabs  responsive-size-type-field__common-tabs  responsive-inline-margin"
                  activeClass="active-tab"
                  tabs={[
                    {
                      name: "desktop",
                      title: <Dashicon icon="desktop" />,
                      className:
                        " responsive-desktop-tab  responsive-responsive-tabs",
                    },
                    {
                      name: "tablet",
                      title: <Dashicon icon="tablet" />,
                      className:
                        " responsive-tablet-tab  responsive-responsive-tabs",
                    },
                    {
                      name: "mobile",
                      title: <Dashicon icon="smartphone" />,
                      className:
                        " responsive-mobile-tab  responsive-responsive-tabs",
                    },
                  ]}
                >
                  {(tab) => {
                    let tabout;

                    if ("mobile" === tab.name) {
                      tabout = (
                        <Fragment>
                          <RangeControl
                            label={__(
                              "Labels Font Size Mobile",
                              "easy-gtblocks"
                            )}
                            min={0}
                            max={500}
                            value={labelFontSizeMobile}
                            onChange={(value) =>
                              setAttributes({
                                labelFontSizeMobile: value,
                              })
                            }
                          />
                        </Fragment>
                      );
                    } else if ("tablet" === tab.name) {
                      tabout = (
                        <Fragment>
                          <RangeControl
                            label={__(
                              "Labels Font Size Tablet",
                              "easy-gtblocks"
                            )}
                            min={0}
                            max={500}
                            value={labelFontSizeTablet}
                            onChange={(value) =>
                              setAttributes({
                                labelFontSizeTablet: value,
                              })
                            }
                          />
                        </Fragment>
                      );
                    } else {
                      tabout = (
                        <Fragment>
                          <RangeControl
                            label={__(
                              "Labels Font Size",
                              "easy-gtblocks"
                            )}
                            min={0}
                            max={500}
                            value={labelFontSize}
                            onChange={(value) =>
                              setAttributes({
                                labelFontSize: value,
                              })
                            }
                          />
                        </Fragment>
                      );
                    }

                    return <div>{tabout}</div>;
                  }}
                </TabPanel>

                <SelectControl
                  label={__(
                    "Labels Font Weight",
                    "easy-gtblocks"
                  )}
                  options={fontWeightOptions}
                  value={labelFontWeight}
                  onChange={(value) =>
                    this.props.setAttributes({
                      labelFontWeight: value,
                    })
                  }
                />

                <RangeControl
                label={__("Labels Line Height", "easy-gtblocks")}
                value={labelFontLineHeight}
                onChange={(value) =>
                  this.props.setAttributes({
                    labelFontLineHeight: value !== undefined ? value : 1,
                  })
                }
                min={0}
                max={100}
                step={1}
                allowReset
                />
              </Fragment>
            )}
            {!hideLabel && (
              <RangeControl
                label={__("Gap between Icon and Label")}
                value={inner_gap}
                onChange={(value) => setAttributes({ inner_gap: value })}
                min={0}
                max={100}
              />
            )}
            <hr className="easy-gtblocks-editor__separator" />
            <SelectControl
              label={__("Icon Alignment")}
              value={iconPosition}
              options={[
                { value: "top", label: __("Top") },
                { value: "middle", label: __("Middle") },
              ]}
              onChange={(value) => setAttributes({ iconPosition: value })}
            />
            <hr className="easy-gtblocks-editor__separator" />
            <RangeControl
              label={__("Icon Size", "easy-gtblocks")}
              value={size}
              onChange={(value) =>
                setAttributes({ size: value !== undefined ? value : 16 })
              }
              min={0}
              max={500}
              allowReset
            />
            <hr className="easy-gtblocks-editor__separator" />
            <RangeControl
              label={__("Background Size", "easy-gtblocks")}
              value={bgSize}
              onChange={(value) => setAttributes({ bgSize: value })}
              help={__(
                "Note: Background Size option is useful when one adds background color to the icons."
              )}
              min={0}
              max={500}
            />
            <RangeControl
              label={__("Border", "easy-gtblocks")}
              value={border}
              onChange={(value) => setAttributes({ border: value })}
              help={__(
                "Note: Border option is useful when one adds border color to the icons."
              )}
              min={0}
              max={10}
            />
            <RangeControl
              label={__("Border Radius", "easy-gtblocks")}
              value={borderRadius}
              onChange={(value) => setAttributes({ borderRadius: value })}
              help={__(
                "Note: Border Radius option is useful when one adds background color to the icons."
              )}
              min={0}
              max={500}
            />
          </PanelBody>
        </InspectorControls>
      </Fragment>
    );
  }
}
