import fontOptions from "../../../utils/googlefonts";
import BoxShadowControl from "../../../utils/components/box-shadow";
import BoxShadowControlHelper from "../../../utils/components/box-shadow-helper";

/**
 * Inspector Controls
 */

// Setup the block
const { __ } = wp.i18n;
const { Component, Fragment } = wp.element;

import compact from "lodash/compact";
import map from "lodash/map";
import { loadGoogleFont } from "../../../utils/font";

// Import block components
const { InspectorControls, ColorPalette } = wp.blockEditor;

// Import Inspector components
const {
  PanelBody,
  QueryControls,
  RangeControl,
  SelectControl,
  TextControl,
  ToggleControl,
  TabPanel,
  Dashicon,
} = wp.components;

const { addQueryArgs } = wp.url;

const { apiFetch } = wp;

const MAX_POSTS_COLUMNS = 4;

/**
 * Create an Inspector Controls wrapper Component
 */
export default class Inspector extends Component {
  constructor() {
    super(...arguments);
    this.state = { categoriesList: [] };
  }

  componentDidMount() {
    this.stillMounted = true;
    this.fetchRequest = apiFetch({
      path: addQueryArgs("/wp/v2/categories", { per_page: -1 }),
    })
      .then((categoriesList) => {
        if (this.stillMounted) {
          this.setState({ categoriesList });
        }
      })
      .catch(() => {
        if (this.stillMounted) {
          this.setState({ categoriesList: [] });
        }
      });
  }

  componentWillUnmount() {
    this.stillMounted = false;
  }

  /* Get the available image sizes */
  imageSizeSelect() {
    const getSettings = wp.data.select("core/editor").getEditorSettings();

    return compact(
      map(getSettings.imageSizes, ({ name, slug }) => {
        return {
          value: slug,
          label: name,
        };
      })
    );
  }

  render() {
    // Setup the attributes
    const { attributes, setAttributes, latestPosts } = this.props;

    const { order, orderBy } = attributes;

    const { categoriesList } = this.state;

    const fontWeightOptions = [
      {
        value: "100",
        label: __("100", "easy-gtblocks"),
      },
      {
        value: "200",
        label: __("200", "easy-gtblocks"),
      },
      {
        value: "300",
        label: __("300", "easy-gtblocks"),
      },
      {
        value: "400",
        label: __("400", "easy-gtblocks"),
      },
      {
        value: "500",
        label: __("500", "easy-gtblocks"),
      },
      {
        value: "600",
        label: __("600", "easy-gtblocks"),
      },
      {
        value: "700",
        label: __("700", "easy-gtblocks"),
      },
      {
        value: "800",
        label: __("800", "easy-gtblocks"),
      },
      {
        value: "900",
        label: __("900", "easy-gtblocks"),
      },
    ];

    const textTransformOptions = [
      {
        value: "",
        label: __("Default", "easy-gtblocks"),
      },
      {
        value: "uppercase",
        label: __("Uppercase", "easy-gtblocks"),
      },
      {
        value: "lowercase",
        label: __("Lowercase", "easy-gtblocks"),
      },
      {
        value: "capitalize",
        label: __("Capitalize", "easy-gtblocks"),
      },
    ];

    const postTaxonomyOptions = [
      {
        value: "category",
        label: __("Category", "easy-gtblocks"),
      },
      { value: "tag", label: __("Tag", "easy-gtblocks") },
    ];

    // Section title tags
    const sectionTags = [
      { value: "div", label: __("div", "easy-gtblocks") },
      {
        value: "header",
        label: __("header", "easy-gtblocks"),
      },
      {
        value: "section",
        label: __("section", "easy-gtblocks"),
      },
      {
        value: "article",
        label: __("article", "easy-gtblocks"),
      },
      { value: "main", label: __("main", "easy-gtblocks") },
      { value: "aside", label: __("aside", "easy-gtblocks") },
      {
        value: "footer",
        label: __("footer", "easy-gtblocks"),
      },
    ];

    // Section title tags
    const sectionTitleTags = [
      { value: "h2", label: __("H2", "easy-gtblocks") },
      { value: "h3", label: __("H3", "easy-gtblocks") },
      { value: "h4", label: __("H4", "easy-gtblocks") },
      { value: "h5", label: __("H5", "easy-gtblocks") },
      { value: "h6", label: __("H6", "easy-gtblocks") },
    ];

    // Check for posts
    const hasPosts = Array.isArray(latestPosts) && latestPosts.length;

    // Check the post type
    const isPost = "post" === attributes.postType;

    // Add instruction text to the select
    const abImageSizeSelect = {
      value: "selectimage",
      label: __("Select image size"),
    };

    // Get the image size options
    const imageSizeOptions = this.imageSizeSelect();

    imageSizeOptions.unshift(abImageSizeSelect);

    const imageSizeValue = () => {
      for (let i = 0; i < imageSizeOptions.length; i++) {
        if (imageSizeOptions[i].value === attributes.imageSize) {
          return attributes.imageSize;
        }
      }
      return "full";
    };

    return (
      <InspectorControls>
        <PanelBody
          title={__(
            "Post and Page Grid Settings",
            "easy-gtblocks"
          )}
          initialOpen={false}
          className={
            isPost ? null : "easy-gtblocks-hide-query"
          }
        >
          <QueryControls
            {...{ order, orderBy }}
            numberOfItems={attributes.postsToShow}
            onNumberOfItemsChange={(value) =>
              setAttributes({ postsToShow: value })
            }
          />
          <SelectControl
            label={__("Order By")}
            value={attributes.orderBy}
            onChange={(value) => setAttributes({ orderBy: value })}
            options={[
              { value: "date", label: __("Date") },
              { value: "title", label: __("Title") },
              { value: "rand", label: __("Random") },
              { value: "menu_order", label: __("Menu Order") },
            ]}
          />
          <SelectControl
            label={__("Order")}
            value={attributes.order}
            onChange={(value) => setAttributes({ order: value })}
            options={[
              { value: "desc", label: __("Descending") },
              { value: "asc", label: __("Ascending") },
            ]}
          />
          <RangeControl
            label={__(
              "Number of items to offset",
              "easy-gtblocks"
            )}
            value={attributes.offset}
            onChange={(value) => setAttributes({ offset: value })}
            min={0}
            max={20}
          />
          {"grid" === attributes.postLayout && (
            <RangeControl
              label={__("Columns", "easy-gtblocks")}
              value={attributes.columns}
              onChange={(value) => setAttributes({ columns: value })}
              min={1}
              max={
                !hasPosts
                  ? MAX_POSTS_COLUMNS
                  : Math.min(MAX_POSTS_COLUMNS, latestPosts.length)
              }
            />
          )}
          <ToggleControl
            label={__("Equal Height", "easy-gtblocks")}
            checked={attributes.equalHeight}
            onChange={() =>
              this.props.setAttributes({
                equalHeight: !attributes.equalHeight,
              })
            }
          />
          <ToggleControl
            label={__("Post Pagination", "easy-gtblocks")}
            checked={attributes.postPagination}
            onChange={() =>
              this.props.setAttributes({
                postPagination: !attributes.postPagination,
                paginationMarkup: "empty",
              })
            }
          />
          <RangeControl
            label={__("Page Limit", "easy-gtblocks")}
            value={attributes.pageLimit}
            onChange={(value) =>
              setAttributes({
                pageLimit: value,
                paginationMarkup: "empty",
              })
            }
            min={0}
            max={100}
          />
        </PanelBody>
        <PanelBody
          title={__("Pagination", "easy-gtblocks")}
          initialOpen={false}
        >
          <SelectControl
            label={__("Pagination Layout", "easy-gtblocks")}
            options={[
              {
                value: "border",
                label: __("Border", "easy-gtblocks"),
              },
              {
                value: "filled",
                label: __("Filled", "easy-gtblocks"),
              },
            ]}
            value={attributes.paginationLayout}
            onChange={(value) =>
              this.props.setAttributes({ paginationLayout: value })
            }
          />
          <SelectControl
            label={__("Pagination Alignment", "easy-gtblocks")}
            options={[
              {
                value: "left",
                label: __("Left", "easy-gtblocks"),
              },
              {
                value: "center",
                label: __("Center", "easy-gtblocks"),
              },
              {
                value: "right",
                label: __("Right", "easy-gtblocks"),
              },
            ]}
            value={attributes.paginationAlignment}
            onChange={(value) =>
              this.props.setAttributes({ paginationAlignment: value })
            }
          />
          <RangeControl
            label={__("Border Size", "easy-gtblocks")}
            value={attributes.paginationBorderWidth}
            onChange={(value) =>
              setAttributes({ paginationBorderWidth: value })
            }
            min={0}
            max={150}
          />
          <p className="responsive-setting-label">
            {__("Color")}
            <span className="components-base-control__label">
              <span
                className="component-color-indicator"
                style={{ backgroundColor: attributes.paginationBorderColor }}
              ></span>
            </span>
          </p>
          <ColorPalette
            value={attributes.paginationBorderColor}
            onChange={(value) =>
              this.props.setAttributes({
                paginationBorderColor: value,
              })
            }
            allowReset
          />
          <RangeControl
            label={__("Border Radius", "easy-gtblocks")}
            value={attributes.paginationBorderRadius}
            onChange={(value) =>
              setAttributes({ paginationBorderRadius: value })
            }
            min={0}
            max={150}
          />
          <p className="responsive-setting-label">
            {__("Active Color")}
            <span className="components-base-control__label">
              <span
                className="component-color-indicator"
                style={{
                  backgroundColor: attributes.paginationActiveBorderColor,
                }}
              ></span>
            </span>
          </p>
          <ColorPalette
            value={attributes.paginationActiveBorderColor}
            onChange={(value) =>
              this.props.setAttributes({
                paginationActiveBorderColor: value,
              })
            }
            allowReset
          />
          <p className="responsive-setting-label">
            {__("Text Color")}
            <span className="components-base-control__label">
              <span
                className="component-color-indicator"
                style={{ backgroundColor: attributes.paginationTextColor }}
              ></span>
            </span>
          </p>
          <ColorPalette
            value={attributes.paginationTextColor}
            onChange={(value) =>
              this.props.setAttributes({
                paginationTextColor: value,
              })
            }
            allowReset
          />
          <p className="responsive-setting-label">
            {__("Active Text Color")}
            <span className="components-base-control__label">
              <span
                className="component-color-indicator"
                style={{
                  backgroundColor: attributes.paginationTextActiveColor,
                }}
              ></span>
            </span>
          </p>
          <ColorPalette
            value={attributes.paginationTextActiveColor}
            onChange={(value) =>
              this.props.setAttributes({
                paginationTextActiveColor: value,
              })
            }
            allowReset
          />
          <RangeControl
            label={__("Spacing", "easy-gtblocks")}
            value={attributes.paginationSpacing}
            onChange={(value) => setAttributes({ paginationSpacing: value })}
            min={0}
            max={500}
          />
          <TextControl
            label={__("Previous Button Text", "easy-gtblocks")}
            type="text"
            value={attributes.previousButtonText}
            onChange={(value) =>
              this.props.setAttributes({
                previousButtonText: value,
                paginationMarkup: "empty",
              })
            }
          />
          <TextControl
            label={__("Next Button Text", "easy-gtblocks")}
            type="text"
            value={attributes.nextButtonText}
            onChange={(value) =>
              this.props.setAttributes({
                nextButtonText: value,
                paginationMarkup: "empty",
              })
            }
          />
        </PanelBody>
        <PanelBody
          title={__(
            "Post and Page Grid Content",
            "easy-gtblocks"
          )}
          initialOpen={false}
        >
          {"list" === attributes.postLayout && (
            <ToggleControl
              label={__(
                "Stack Image & Content on mobile",
                "easy-gtblocks"
              )}
              checked={attributes.stackonMobile}
              onChange={() =>
                this.props.setAttributes({
                  stackonMobile: !attributes.stackonMobile,
                })
              }
            />
          )}
          <ToggleControl
            label={__(
              "Display Section Title",
              "easy-gtblocks"
            )}
            checked={attributes.displaySectionTitle}
            onChange={() =>
              this.props.setAttributes({
                displaySectionTitle: !attributes.displaySectionTitle,
              })
            }
          />
          {attributes.displaySectionTitle && (
            <TextControl
              label={__("Section Title", "easy-gtblocks")}
              type="text"
              value={attributes.sectionTitle}
              onChange={(value) =>
                this.props.setAttributes({
                  sectionTitle: value,
                })
              }
            />
          )}
          <ToggleControl
            label={__(
              "Display Featured Image",
              "easy-gtblocks"
            )}
            checked={attributes.displayPostImage}
            onChange={() =>
              this.props.setAttributes({
                displayPostImage: !attributes.displayPostImage,
              })
            }
          />
          {attributes.displayPostImage && [
            <SelectControl
              label={__("Image Size", "easy-gtblocks")}
              value={imageSizeValue()}
              options={imageSizeOptions}
              onChange={(value) =>
                this.props.setAttributes({
                  imageSize: value,
                })
              }
            />,
            <SelectControl
              label={__("Image Position", "easy-gtblocks")}
              value={attributes.imagePosition}
              options={[
                {
                  value: "top",
                  label: __("Top", "easy-gtblocks"),
                },
                {
                  value: "background",
                  label: __("Background", "easy-gtblocks"),
                },
              ]}
              onChange={(value) =>
                this.props.setAttributes({
                  imagePosition: value,
                })
              }
            />,
            <SelectControl
            label={__("Layout", "easy-gtblocks")}
            value={attributes.layout}
            options={[
              {
                value: "boxed",
                label: __("Boxed", "easy-gtblocks"),
              },
              {
                value: "content",
                label: __("Content", "easy-gtblocks"),
              },
            ]}
            onChange={(value) =>
              this.props.setAttributes({
                layout: value,
              })
            }
            />,
            <RangeControl
              label={__("Image Border Radius")}
              value={attributes.imageBorderRadius}
              onChange={(value) =>
                this.props.setAttributes({
                  imageBorderRadius: value,
                })
              }
              min={0}
              max={100}
            />,
          ]}
          <ToggleControl
            label={__("Display Title", "easy-gtblocks")}
            checked={attributes.displayPostTitle}
            onChange={() =>
              this.props.setAttributes({
                displayPostTitle: !attributes.displayPostTitle,
              })
            }
          />
          {isPost && (
            <ToggleControl
              label={__("Display Author", "easy-gtblocks")}
              checked={attributes.displayPostAuthor}
              onChange={() =>
                this.props.setAttributes({
                  displayPostAuthor: !attributes.displayPostAuthor,
                })
              }
            />
          )}
          {isPost && (
            <ToggleControl
              label={__("Display Date", "easy-gtblocks")}
              checked={attributes.displayPostDate}
              onChange={() =>
                this.props.setAttributes({
                  displayPostDate: !attributes.displayPostDate,
                })
              }
            />
          )}
          <ToggleControl
            label={__("Display Excerpt", "easy-gtblocks")}
            checked={attributes.displayPostExcerpt}
            onChange={() =>
              this.props.setAttributes({
                displayPostExcerpt: !attributes.displayPostExcerpt,
              })
            }
          />
          {attributes.displayPostExcerpt && (
            <RangeControl
              label={__("Excerpt Length", "easy-gtblocks")}
              value={attributes.excerptLength}
              onChange={(value) => setAttributes({ excerptLength: value })}
              min={0}
              max={55}
            />
          )}
          <ToggleControl
            label={__(
              "Display Continue Reading Link",
              "easy-gtblocks"
            )}
            checked={attributes.displayPostLink}
            onChange={() =>
              this.props.setAttributes({
                displayPostLink: !attributes.displayPostLink,
              })
            }
          />
          {attributes.displayPostLink && (
            <TextControl
              label={__(
                "Customize Continue Reading Text",
                "easy-gtblocks"
              )}
              type="text"
              value={attributes.readMoreText}
              onChange={(value) =>
                this.props.setAttributes({
                  readMoreText: value,
                })
              }
            />
          )}
        </PanelBody>
        <PanelBody
          title={__(
            "Post and Page Grid Markup",
            "easy-gtblocks"
          )}
          initialOpen={false}
          className="easy-gtblocks-block-post-grid-markup-settings"
        >
          <SelectControl
            label={__(
              "Post Grid Section Tag",
              "easy-gtblocks"
            )}
            options={sectionTags}
            value={attributes.sectionTag}
            onChange={(value) =>
              this.props.setAttributes({ sectionTag: value })
            }
            help={__(
              "Change the post grid section tag to match your content hierarchy.",
              "easy-gtblocks"
            )}
          />
          {attributes.sectionTitle && (
            <SelectControl
              label={__(
                "Section Title Heading Tag",
                "easy-gtblocks"
              )}
              options={sectionTitleTags}
              value={attributes.sectionTitleTag}
              onChange={(value) =>
                this.props.setAttributes({
                  sectionTitleTag: value,
                })
              }
              help={__(
                "Change the post/page section title tag to match your content hierarchy.",
                "easy-gtblocks"
              )}
            />
          )}
          {attributes.displayPostTitle && (
            <SelectControl
              label={__(
                "Post Title Heading Tag",
                "easy-gtblocks"
              )}
              options={sectionTitleTags}
              value={attributes.postTitleTag}
              onChange={(value) =>
                this.props.setAttributes({
                  postTitleTag: value,
                })
              }
              help={__(
                "Change the post/page title tag to match your content hierarchy.",
                "easy-gtblocks"
              )}
            />
          )}
        </PanelBody>
        <PanelBody
          title={__("Typography", "easy-gtblocks")}
          initialOpen={false}
        >
          <PanelBody
            title={__("Excerpt", "easy-gtblocks")}
            initialOpen={false}
          >
            <SelectControl
              label={__("Font Family", "easy-gtblocks")}
              options={fontOptions}
              value={attributes.excerptFontFamily}
              onChange={(value) => {
                this.props.setAttributes({
                  excerptFontFamily: value,
                }),
                  loadGoogleFont(value);
              }}
            />
            <RangeControl
              label={__("Font Size", "easy-gtblocks")}
              value={attributes.excerptFontSize}
              onChange={(value) =>
                this.props.setAttributes({
                  excerptFontSize: value,
                })
              }
              min={0}
              max={50}
              step={1}
            />
            <SelectControl
              label={__("Font Weight", "easy-gtblocks")}
              options={fontWeightOptions}
              value={attributes.excerptFontWeight}
              onChange={(value) =>
                this.props.setAttributes({
                  excerptFontWeight: value,
                })
              }
            />
            <RangeControl
              label={__("Line Height", "easy-gtblocks")}
              value={attributes.excerptLineHeight}
              onChange={(value) =>
                this.props.setAttributes({
                  excerptLineHeight: value,
                })
              }
              min={0}
              max={100}
              step={1}
            />
            <SelectControl
              label={__("Text Transform", "easy-gtblocks")}
              options={textTransformOptions}
              value={attributes.excerptTextTransform}
              onChange={(value) =>
                this.props.setAttributes({
                  excerptTextTransform: value,
                })
              }
            />
          </PanelBody>
          <PanelBody
            title={__("Meta", "easy-gtblocks")}
            initialOpen={false}
          >
            <SelectControl
              label={__("Font Family", "easy-gtblocks")}
              options={fontOptions}
              value={attributes.metaFontFamily}
              onChange={(value) => {
                this.props.setAttributes({
                  metaFontFamily: value,
                }),
                  loadGoogleFont(value);
              }}
            />
            <RangeControl
              label={__("Font Size", "easy-gtblocks")}
              value={attributes.metaFontSize}
              onChange={(value) =>
                this.props.setAttributes({
                  metaFontSize: value,
                })
              }
              min={0}
              max={50}
              step={1}
            />
            <SelectControl
              label={__("Font Weight", "easy-gtblocks")}
              options={fontWeightOptions}
              value={attributes.metaFontWeight}
              onChange={(value) =>
                this.props.setAttributes({
                  metaFontWeight: value,
                })
              }
            />
            <RangeControl
              label={__("Line Height", "easy-gtblocks")}
              value={attributes.metaLineHeight}
              onChange={(value) =>
                this.props.setAttributes({
                  metaLineHeight: value,
                })
              }
              min={0}
              max={100}
              step={1}
            />
            <SelectControl
              label={__("Text Transform", "easy-gtblocks")}
              options={textTransformOptions}
              value={attributes.metaTextTransform}
              onChange={(value) =>
                this.props.setAttributes({
                  metaTextTransform: value,
                })
              }
            />
          </PanelBody>
          <PanelBody
            title={__("Title", "easy-gtblocks")}
            initialOpen={false}
          >
            <SelectControl
              label={__("Font Family", "easy-gtblocks")}
              options={fontOptions}
              value={attributes.titleFontFamily}
              onChange={(value) => {
                this.props.setAttributes({
                  titleFontFamily: value,
                }),
                  loadGoogleFont(value);
              }}
            />
            <TabPanel
              className=" responsive-size-type-field-tabs  responsive-size-type-field__common-tabs  responsive-inline-margin"
              activeClass="active-tab"
              tabs={[
                {
                  name: "desktop",
                  title: <Dashicon icon="desktop" />,
                  className:
                    " responsive-desktop-tab  responsive-responsive-tabs",
                },
                {
                  name: "tablet",
                  title: <Dashicon icon="tablet" />,
                  className:
                    " responsive-tablet-tab  responsive-responsive-tabs",
                },
                {
                  name: "mobile",
                  title: <Dashicon icon="smartphone" />,
                  className:
                    " responsive-mobile-tab  responsive-responsive-tabs",
                },
              ]}
            >
              {(tab) => {
                let tabout;

                if ("mobile" === tab.name) {
                  tabout = (
                    <Fragment>
                      <RangeControl
                        label={__(
                          "Font Size",
                          "easy-gtblocks"
                        )}
                        min={0}
                        max={500}
                        value={attributes.titleFontSizeMobile}
                        onChange={(value) =>
                          setAttributes({
                            titleFontSizeMobile: value,
                          })
                        }
                      />
                    </Fragment>
                  );
                } else if ("tablet" === tab.name) {
                  tabout = (
                    <Fragment>
                      <RangeControl
                        label={__(
                          "Font Size",
                          "easy-gtblocks"
                        )}
                        min={0}
                        max={500}
                        value={attributes.titleFontSizeTablet}
                        onChange={(value) =>
                          setAttributes({
                            titleFontSizeTablet: value,
                          })
                        }
                      />
                    </Fragment>
                  );
                } else {
                  tabout = (
                    <Fragment>
                      <RangeControl
                        label={__(
                          "Font Size",
                          "easy-gtblocks"
                        )}
                        min={0}
                        max={500}
                        value={attributes.titleFontSize}
                        onChange={(value) =>
                          setAttributes({
                            titleFontSize: value,
                          })
                        }
                      />
                    </Fragment>
                  );
                }

                return <div>{tabout}</div>;
              }}
            </TabPanel>
            <SelectControl
              label={__("Font Weight", "easy-gtblocks")}
              options={fontWeightOptions}
              value={attributes.titleFontWeight}
              onChange={(value) =>
                this.props.setAttributes({
                  titleFontWeight: value,
                })
              }
            />
            <RangeControl
              label={__("Line Height", "easy-gtblocks")}
              value={attributes.titleLineHeight}
              onChange={(value) =>
                this.props.setAttributes({
                  titleLineHeight: value,
                })
              }
              min={0}
              max={100}
              step={1}
            />
            <SelectControl
              label={__("Text Transform", "easy-gtblocks")}
              options={textTransformOptions}
              value={attributes.titleTextTransform}
              onChange={(value) =>
                this.props.setAttributes({
                  titleTextTransform: value,
                })
              }
            />
          </PanelBody>
          <PanelBody
            title={__("Read More Link", "easy-gtblocks")}
            initialOpen={false}
          >
            <RangeControl
              label={__("Font Size", "easy-gtblocks")}
              min={0}
              max={500}
              value={attributes.continueFontSize}
              onChange={(value) =>
                setAttributes({
                  continueFontSize: value,
                })
              }
            />
            <SelectControl
              label={__("Font Weight", "easy-gtblocks")}
              options={fontWeightOptions}
              value={attributes.continueFontWeight}
              onChange={(value) =>
                this.props.setAttributes({
                  continueFontWeight: value,
                })
              }
            />
            <RangeControl
              label={__("Line Height", "easy-gtblocks")}
              value={attributes.continueLineHeight}
              onChange={(value) =>
                this.props.setAttributes({
                  continueLineHeight: value,
                })
              }
              min={0}
              max={100}
              step={1}
            />
            <SelectControl
              label={__("Text Transform", "easy-gtblocks")}
              options={textTransformOptions}
              value={attributes.continueTextTransform}
              onChange={(value) =>
                this.props.setAttributes({
                  continueTextTransform: value,
                })
              }
            />
          </PanelBody>
        </PanelBody>
        <PanelBody
          title={__("Border", "easy-gtblocks")}
          initialOpen={false}
        >
          <SelectControl
            label={__("Border Style")}
            value={attributes.blockBorderStyle}
            onChange={(value) =>
              this.props.setAttributes({ blockBorderStyle: value })
            }
            options={[
              { value: "none", label: __("None") },
              { value: "solid", label: __("Solid") },
              { value: "dotted", label: __("Dotted") },
              { value: "dashed", label: __("Dashed") },
              { value: "double", label: __("Double") },
              { value: "groove", label: __("Groove") },
              { value: "inset", label: __("Inset") },
              { value: "outset", label: __("Outset") },
              { value: "ridge", label: __("Ridge") },
            ]}
          />
          {"none" != attributes.blockBorderStyle && (
            <RangeControl
              label={__("Border Width")}
              value={attributes.blockBorderWidth}
              onChange={(value) =>
                this.props.setAttributes({
                  blockBorderWidth: value !== undefined ? value : 1,
                })
              }
              min={0}
              max={50}
              allowReset
            />
          )}
          <RangeControl
            label={__("Border Radius")}
            value={attributes.blockBorderRadius}
            onChange={(value) =>
              this.props.setAttributes({
                blockBorderRadius: value !== undefined ? value : "",
              })
            }
            min={0}
            max={1000}
            allowReset
          />
          {"none" != attributes.blockBorderStyle && (
            <Fragment>
              <p className="responsive-setting-label">
                {__("Border Color")}
                <span className="components-base-control__label">
                  <span
                    className="component-color-indicator"
                    style={{ backgroundColor: attributes.blockBorderColor }}
                  ></span>
                </span>
              </p>
              <ColorPalette
                value={attributes.blockBorderColor}
                onChange={(colorValue) =>
                  this.props.setAttributes({ blockBorderColor: colorValue })
                }
                allowReset
              />
            </Fragment>
          )}
          <BoxShadowControl
            setAttributes={setAttributes}
            label={__("Box Shadow")}
            boxShadowColor={{
              value: attributes.boxShadowColor,
              label: __("Color"),
            }}
            boxShadowHOffset={{
              value: attributes.boxShadowHOffset,
              label: __("Horizontal"),
            }}
            boxShadowVOffset={{
              value: attributes.boxShadowVOffset,
              label: __("Vertical"),
            }}
            boxShadowBlur={{
              value: attributes.boxShadowBlur,
              label: __("Blur"),
            }}
            boxShadowSpread={{
              value: attributes.boxShadowSpread,
              label: __("Spread"),
            }}
            boxShadowPosition={{
              value: attributes.boxShadowPosition,
              label: __("Position"),
            }}
          />
          <BoxShadowControlHelper
            setAttributes={setAttributes}
            label={__("Hover Box Shadow")}
            attrNameTemplate="hover%s"
      boxShadowColor={{ value: attributes.hoverboxShadowColor }}
          />
        </PanelBody>
        <PanelBody
          title={__("Color", "easy-gtblocks")}
          initialOpen={false}
        >
          <p className="responsive-setting-label">
            {__("Background")}
            <span className="components-base-control__label">
              <span
                className="component-color-indicator"
                style={{ backgroundColor: attributes.bgColor }}
              ></span>
            </span>
          </p>
          <ColorPalette
            value={attributes.bgColor}
            onChange={(value) =>
              this.props.setAttributes({
                bgColor: value,
              })
            }
            allowReset
          />
          <p className="responsive-setting-label">
            {__("Title Color")}
            <span className="components-base-control__label">
              <span
                className="component-color-indicator"
                style={{
                  backgroundColor: attributes.titleColor,
                }}
              ></span>
            </span>
          </p>
          <ColorPalette
            value={attributes.titleColor}
            onChange={(value) =>
              this.props.setAttributes({
                titleColor: value,
              })
            }
            allowReset
          />
          <p className="responsive-setting-label">
            {__("Title Hover Color")}
            <span className="components-base-control__label">
              <span
                className="component-color-indicator"
                style={{
                  backgroundColor: attributes.titleHoverColor,
                }}
              ></span>
            </span>
          </p>
          <ColorPalette
            value={attributes.titleHoverColor}
            onChange={(value) =>
              this.props.setAttributes({
                titleHoverColor: value,
              })
            }
            allowReset
          />
          <p className="responsive-setting-label">
            {__("Meta Color")}
            <span className="components-base-control__label">
              <span
                className="component-color-indicator"
                style={{
                  backgroundColor: attributes.metaColor,
                }}
              ></span>
            </span>
          </p>
          <ColorPalette
            value={attributes.metaColor}
            onChange={(value) =>
              this.props.setAttributes({
                metaColor: value,
              })
            }
            allowReset
          />
          <p className="responsive-setting-label">
            {__("Excerpt Color")}
            <span className="components-base-control__label">
              <span
                className="component-color-indicator"
                style={{
                  backgroundColor: attributes.textColor,
                }}
              ></span>
            </span>
          </p>
          <ColorPalette
            value={attributes.textColor}
            onChange={(value) =>
              this.props.setAttributes({
                textColor: value,
              })
            }
            allowReset
          />
          <p className="responsive-setting-label">
            {__("Read More Link Color")}
            <span className="components-base-control__label">
              <span
                className="component-color-indicator"
                style={{
                  backgroundColor: attributes.readMoreLinkColor,
                }}
              ></span>
            </span>
          </p>
          <ColorPalette
            value={attributes.readMoreLinkColor}
            onChange={(value) =>
              this.props.setAttributes({
                readMoreLinkColor: value,
              })
            }
            allowReset
          />
        </PanelBody>
        <PanelBody
          title={__("Spacing", "easy-gtblocks")}
          initialOpen={false}
        >
          <TabPanel
            className=" responsive-size-type-field-tabs  responsive-size-type-field__common-tabs  responsive-inline-margin"
            activeClass="active-tab"
            tabs={[
              {
                name: "desktop",
                title: <Dashicon icon="desktop" />,
                className:
                  " responsive-desktop-tab  responsive-responsive-tabs",
              },
              {
                name: "tablet",
                title: <Dashicon icon="tablet" />,
                className: " responsive-tablet-tab  responsive-responsive-tabs",
              },
              {
                name: "mobile",
                title: <Dashicon icon="smartphone" />,
                className: " responsive-mobile-tab  responsive-responsive-tabs",
              },
            ]}
          >
            {(tab) => {
              let tabout;

              if ("mobile" === tab.name) {
                tabout = (
                  <Fragment>
                    <RangeControl
                      label={__("Row Gap", "atomic-blocks")}
                      value={attributes.rowGapMobile}
                      onChange={(value) =>
                        this.props.setAttributes({
                          rowGapMobile: value,
                        })
                      }
                      min={0}
                      max={50}
                      step={1}
                    />
                  </Fragment>
                );
              } else if ("tablet" === tab.name) {
                tabout = (
                  <Fragment>
                    <RangeControl
                      label={__("Row Gap Tablet", "atomic-blocks")}
                      value={attributes.rowGapTablet}
                      onChange={(value) =>
                        this.props.setAttributes({
                          rowGapTablet: value,
                        })
                      }
                      min={0}
                      max={50}
                      step={1}
                    />
                  </Fragment>
                );
              } else {
                tabout = (
                  <Fragment>
                    <RangeControl
                      label={__("Row Gap", "atomic-blocks")}
                      value={attributes.rowGap}
                      onChange={(value) =>
                        this.props.setAttributes({
                          rowGap: value,
                        })
                      }
                      min={0}
                      max={50}
                      step={1}
                    />
                  </Fragment>
                );
              }

              return <div>{tabout}</div>;
            }}
          </TabPanel>
          <RangeControl
            label={__("Column Gap", "atomic-blocks")}
            value={attributes.columnGap}
            onChange={(value) =>
              this.props.setAttributes({
                columnGap: value,
              })
            }
            min={0}
            max={50}
            step={1}
          />
          <RangeControl
            label={__("Content Padding", "atomic-blocks")}
            value={attributes.contentPadding}
            onChange={(value) =>
              this.props.setAttributes({
                contentPadding: value,
              })
            }
            min={0}
            max={100}
            step={1}
          />
          <RangeControl
            label={__("Mobile Content Padding", "atomic-blocks")}
            value={attributes.mobileContentPadding}
            onChange={(value) =>
              this.props.setAttributes({
                mobileContentPadding: value,
              })
            }
            min={0}
            max={100}
            step={1}
          />
          <RangeControl
            label={__("Title Bottom Spacing", "atomic-blocks")}
            value={attributes.titleBottomSpacing}
            onChange={(value) =>
              this.props.setAttributes({
                titleBottomSpacing: value,
              })
            }
            min={0}
            max={50}
            step={1}
          />
          <RangeControl
            label={__("Meta Bottom Spacing", "atomic-blocks")}
            value={attributes.metaBottomSpacing}
            onChange={(value) =>
              this.props.setAttributes({
                metaBottomSpacing: value,
              })
            }
            min={0}
            max={50}
            step={1}
          />
          <RangeControl
            label={__("Read More Top Spacing", "atomic-blocks")}
            value={attributes.readMoreTopSpacing}
            onChange={(value) =>
              this.props.setAttributes({
                readMoreTopSpacing: value,
              })
            }
            min={0}
            max={50}
            step={1}
          />
        </PanelBody>
      </InspectorControls>
    );
  }
}
